"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
var _appRouteRouteMatcher = require("../../route-matchers/app-route-route-matcher");
var _normalizers = require("../../normalizers/normalizers");
var _absoluteFilenameNormalizer = require("../../normalizers/absolute-filename-normalizer");
var _wrapNormalizerFn = require("../../normalizers/wrap-normalizer-fn");
var _appPaths = require("../../../../shared/lib/router/utils/app-paths");
var _prefixingNormalizer = require("../../normalizers/prefixing-normalizer");
var _routeKind = require("../../route-kind");
var _fileCacheRouteMatcherProvider = require("./file-cache-route-matcher-provider");
var _isAppRouteRoute = require("../../../../lib/is-app-route-route");
class DevAppRouteRouteMatcherProvider extends _fileCacheRouteMatcherProvider.FileCacheRouteMatcherProvider {
    constructor(appDir, extensions, reader){
        super(appDir, reader);
        const pageNormalizer = new _absoluteFilenameNormalizer.AbsoluteFilenameNormalizer(appDir, extensions);
        this.normalizers = {
            page: pageNormalizer,
            pathname: new _normalizers.Normalizers([
                pageNormalizer,
                // The pathname to match should have the trailing `/route` and other route
                // group information stripped from it.
                (0, _wrapNormalizerFn).wrapNormalizerFn(_appPaths.normalizeAppPath), 
            ]),
            bundlePath: new _normalizers.Normalizers([
                pageNormalizer,
                // Prefix the bundle path with `app/`.
                new _prefixingNormalizer.PrefixingNormalizer("app"), 
            ])
        };
    }
    async transform(files) {
        const matchers = [];
        for (const filename of files){
            const page = this.normalizers.page.normalize(filename);
            // If the file isn't a match for this matcher, then skip it.
            if (!(0, _isAppRouteRoute).isAppRouteRoute(page)) continue;
            const pathname = this.normalizers.pathname.normalize(filename);
            const bundlePath = this.normalizers.bundlePath.normalize(filename);
            matchers.push(new _appRouteRouteMatcher.AppRouteRouteMatcher({
                kind: _routeKind.RouteKind.APP_ROUTE,
                pathname,
                page,
                bundlePath,
                filename
            }));
        }
        return matchers;
    }
}
exports.DevAppRouteRouteMatcherProvider = DevAppRouteRouteMatcherProvider;

//# sourceMappingURL=dev-app-route-route-matcher-provider.js.map