import { LocaleRouteDefinition } from '../route-definitions/locale-route-definition';
import { LocaleRouteMatch } from '../route-matches/locale-route-match';
import { RouteMatcher } from './route-matcher';
export declare type LocaleMatcherMatchOptions = {
    /**
     * If defined, this indicates to the matcher that the request should be
     * treated as locale-aware. If this is undefined, it means that this
     * application was not configured for additional locales.
     */
    i18n?: {
        /**
         * The locale that was detected on the incoming route. If `undefined` it
         * means that the locale should be considered to be the default one.
         *
         * For example, if the default locale is `en` and the incoming route is
         * `/about`, then this would be `undefined`. If the incoming route was
         * `/en/about`, then this would be `en`. If the incoming route was
         * `/fr/about` then this would be `fr`.
         */
        detectedLocale?: string;
        /**
         * The pathname that has had it's locale information stripped from it.
         *
         * For example, if the pathname previous was `/en/about` and the locale was
         * `en`, then this would be `/about`.
         */
        pathname: string;
    };
};
export declare class LocaleRouteMatcher<D extends LocaleRouteDefinition = LocaleRouteDefinition> extends RouteMatcher<D> {
    /**
     * Identity returns the identity part of the matcher. This is used to compare
     * a unique matcher to another. This is also used when sorting dynamic routes,
     * so it must contain the pathname part as well.
     */
    get identity(): string;
    /**
     * Match will attempt to match the given pathname against this route while
     * also taking into account the locale information.
     *
     * @param pathname The pathname to match against.
     * @param options The options to use when matching.
     * @returns The match result, or `null` if there was no match.
     */
    match(pathname: string, options?: LocaleMatcherMatchOptions): LocaleRouteMatch<D> | null;
    /**
     * Test will attempt to match the given pathname against this route while
     * also taking into account the locale information.
     *
     * @param pathname The pathname to match against.
     * @param options The options to use when matching.
     * @returns The match result, or `null` if there was no match.
     */
    test(pathname: string, options?: LocaleMatcherMatchOptions): {
        params?: import("../../../shared/lib/router/utils/route-matcher").Params | undefined;
    } | null;
}
